@@ -48,6 +48,8 @@ class _Player(object):
         raise NotImplemented
     def stop(self):
         raise NotImplemented
+    def seek(self, nanoseconds=None, percent=None):
+        raise NotImplemented
     def set_eos_callback(self, cb):
         raise NotImplemented
 
@@ -110,12 +112,15 @@ class GStreamer(_Player):
         return 'none'
 
     def get_position_duration(self):
-        try:
-            pos_int = self.player.query_position(self.time_format, None)[0]
-            dur_int = self.player.query_duration(self.time_format, None)[0]
-        except Exception, e:
-            log.exception('Error getting position')
-            pos_int = dur_int = 0
+        pos_int, dur_int = 0, 0
+        if self.player:
+            try:
+                if self.player.get_state()[1] != gst.STATE_NULL:
+                    pos_int = self.player.query_position(self.time_format, None)[0]
+                    dur_int = self.player.query_duration(self.time_format, None)[0]
+            except Exception, e:
+                log.exception('Error getting position')
+                pos_int, dur_int = 0, 0
         return pos_int, dur_int
 
     def playing(self):
@@ -185,6 +190,24 @@ class GStreamer(_Player):
             self.filesrc.set_property(self.filesrc_property, uri)
             log.info("%s", uri)
 
+    def seek(self, nanoseconds=None, percent=None):
+        """if nanoseconds, seek nanoseconds from now
+        if percent, seek percent from start of file
+        """
+        try:
+            position, duration = self.get_position_duration()
+            if not (position or duration) or self.get_state() == 'stopped':
+                return
+            if nanoseconds is not None:
+                position = max(0, min(position+nanoseconds, duration))
+            elif percent is not None:
+                position = int(duration*percent)
+            else:
+                return
+            self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, position)
+        except Exception, e:
+            log.exception( 'Error seeking' )
+
     def _on_message(self, bus, message):
         t = message.type
 
@@ -416,10 +439,6 @@ class Player(object):
             self._play_track(entry, notify='next')
         elif self.playlist.radio_mode:
             self._refill_radio()
-            #if self.playlist.has_next():
-            #    self.next()
-            #else:
-            #    self.stop()
         else:
             self.stop()
 
@@ -440,6 +459,9 @@ class Player(object):
     def playing(self):
         return self.backend.playing()
 
+    def seek(self, nanoseconds=None, percent=None):
+        self.backend.seek(nanoseconds=nanoseconds, percent=percent)
+
     def _on_eos(self):
         self.next()
 
