@@ -21,6 +21,7 @@
 #  Copyright (c) 2008-05-26 Thomas Perl <thpinfo.com>
 #  (based on http://pygstdocs.berlios.de/pygst-tutorial/seeking.html)
 #
+import os
 import gtk
 import gobject
 try:
@@ -28,7 +29,6 @@ try:
 except:
     import helldon as hildon
 import util
-import datetime
 import pango
 import jamaendo
 from settings import settings
@@ -38,7 +38,6 @@ import logging
 import cgi
 
 from songposition import SongPosition
-from listbox import ListDialog
 import colors
 log = logging.getLogger(__name__)
 
@@ -80,6 +79,7 @@ class PlayerWindow(hildon.StackableWindow):
         self.playtime.set_alignment(1,0)
         self.playtime.set_ellipsize(pango.ELLIPSIZE_END)
         self.progress = SongPosition()
+        self.progress.connect('button-press-event', self.on_progress_clicked)
 
         self.set_labels('', '', '', 0, 0)
 
@@ -89,8 +89,8 @@ class PlayerWindow(hildon.StackableWindow):
         vbox2.pack_start(self.track, True)
         vbox2.pack_start(self.artist, True)
         vbox2.pack_start(self.album, True)
-        vbox2.pack_start(self.playtime, False, True)
         vbox2.pack_start(self.progress, False, True)
+        vbox2.pack_start(self.playtime, False, True)
 
         hbox.set_border_width(8)
         hbox.pack_start(self.cover, False, True, 8)
@@ -115,9 +115,8 @@ class PlayerWindow(hildon.StackableWindow):
         postoffice.connect('album-cover', self, self.set_album_cover)
         postoffice.connect(['next', 'prev', 'play', 'pause', 'stop'], self, self.on_state_changed)
 
-        self.on_state_changed()
-
         self.create_menu()
+        self.update_everything()
 
     def create_menu(self):
         self.menu = hildon.AppMenu()
@@ -161,7 +160,8 @@ class PlayerWindow(hildon.StackableWindow):
         self.menu.show_all()
         self.set_app_menu(self.menu)
 
-    def on_state_changed(self, *args):
+
+    def update_everything(self):
         self.update_state()
         self.update_play_button()
 
@@ -170,6 +170,8 @@ class PlayerWindow(hildon.StackableWindow):
         else:
             self.stop_position_timer()
 
+    def on_state_changed(self, *args):
+        gobject.idle_add(self.update_everything)
 
     def get_album_id(self):
         if self.player.playlist and self.player.playlist.current():
@@ -258,11 +260,18 @@ class PlayerWindow(hildon.StackableWindow):
         else:
             settings.volume = widget.get_level()/100.0
 
+    def on_progress_clicked(self, widget, event):
+        w = widget.allocation.width
+        if w > 0:
+            def seeker(percent):
+                the_player.seek(percent=percent)
+            gobject.idle_add(seeker, float(event.x)/float(w))
+
     def on_position_timeout(self):
-        if the_player.playing():
-            self.set_song_position(*the_player.get_position_duration())
-        else:
-            log.debug("position timeout, but not playing")
+        def updater(pos, dur):
+            if the_player.playing():
+                self.set_song_position(pos, dur)
+        gobject.idle_add(updater, *the_player.get_position_duration())
         return True
 
     def start_position_timer(self):
@@ -303,12 +312,6 @@ class PlayerWindow(hildon.StackableWindow):
              )
         self.playtime.set_markup(txt)
 
-    def set_default_cover(self):
-        tmp = util.find_resource('album.png')
-        if tmp:
-            log.debug("Setting cover to %s", tmp)
-            self.cover.set_from_file(tmp)
-
     def update_state(self):
         item = self.playlist.current()
         if item:
@@ -327,12 +330,36 @@ class PlayerWindow(hildon.StackableWindow):
             if isinstance(btn, hildon.GtkButton):
                 btn.set_sensitive(state)
 
+    def get_pixbuf(self, img):
+        try:
+            return gtk.gdk.pixbuf_new_from_file(img)
+        except gobject.GError:
+            log.error("Broken image in cache: %s", img)
+            try:
+                os.unlink(img)
+            except OSError, e:
+                log.warning("Failed to unlink broken image.")
+            if img != self.default_pixbuf:
+                return self.get_default_pixbuf()
+            else:
+                return None
+
+    def set_default_cover(self):
+        tmp = util.find_resource('album.png')
+        if tmp:
+            log.debug("Setting cover to %s", tmp)
+            pb = self.get_pixbuf(tmp)
+            if pb:
+                self.cover.set_from_pixbuf(pb)
+
     def set_album_cover(self, albumid, size, cover):
         if size == 300:
             playing = self.get_album_id()
             if playing and albumid and (int(playing) == int(albumid)):
                 log.debug("Setting cover to %s", cover)
-                self.cover.set_from_file(cover)
+                pb = self.get_pixbuf(cover)
+                if pb:
+                    self.cover.set_from_pixbuf(pb)
 
     def play_radio(self, radio_name, radio_id):
         playlist = Playlist([])
@@ -341,9 +368,6 @@ class PlayerWindow(hildon.StackableWindow):
         playlist.radio_id = radio_id
         hildon.hildon_gtk_window_set_progress_indicator(self, 1)
         self.__play_tracks(playlist)
-        #log.debug("Playlist current: %s, playing? %s",
-        #          playlist.current_index(),
-        #          self.player.playing())
 
     def play_tracks(self, tracks):
         self.__play_tracks(tracks)
